// B_PM20_Wander.ino
/* B_PM20_Wander sketch group is functionally identical to B_PM19SpeedWander.ino
   The sketch was getting long and now all the basic functions are there
   it has been split into sections with each section in a separate file.
   When B_PM20_Wander, the main sketch file, is loaded into the Arduino IDE
   it loads the other files into Tabs.
   The declarations have been moved to .h Header files which at compile time 
   the IDE includes at the #include "filename.h" statements,
   - note that these simple header files use "" unlike Libraries which use <>.
   Also an unchanging part of Setup() has been moved to Init.h. 
   The functions have been moved to .ino files which at compile time the IDE adds,
   in alphabetical order, to the end of this sketch file. 
   It is not advisable to put declarations of pins, constants or variables in 
   those .ino files since the declarations must be before where they are referenced.
   So keep declarations in the .h Header files.
*/

char thisrobot[] ="BambinoM ";
char thisprog[]  ="B_PM20_Wander";
// Binary sketch size: 6,198 bytes (of a 30,720 byte maximum)

//== SECTIONS ===============================================
//== NEW 
//== CHANGED
//== INCLUDES
//     pinsPM20.h     - pin usage
//     varsConsPM20.h - EEPROM addresses, general, Ultrasonic module HC-SR04
//     EEPROM.h       - the Arduino EEPROM library
//== EEPROM
//== SETUP
//     initPM20.h     - Initialise various things
//== LOOP
//TAB FILES
//   actsPM20.ino
//     ACTS - Wander()
//   behaviourPM20.ino
//     BEHAVIOUR MODIFIERS - speeddown(), speedup()
//     BEHAVIOUR RULES - avoidUObjects(), avoidObstacles(), avoidHoles()
//   sensePM20.ino
//     SENSE - dist_HC_SR04()
//           - readWhiskers(), QueryObstacle(), showWhiskerLightLevels() 
//           - readBeard(), QueryEdge(), showBeardLightLevels() 
//   movement20.ino
//     META MOVEMENT - turnAway(), turnMemory(), turnOther() 
//     MOVEMENT - forward(), backward(), turnRight(), turnLeft(),
//                rollCentre(), rollRight(), rollLeft(), 
//                paceCentre(), paceRightForward(), paceLeftForward(), paceRightBackward(), paceLeftBackward(), 
//                rollmove(), pacemove()
//   servos20.ino
//     SERVOS - ServosInitC()
//   hardware20.ino
//     HARDWARE - ServoPulse(byte Spin, int Spulse), beep(byte beeppin,int beepfreqHz,long beeptime)
//   system20.ino
//     SYSTEM - toggle_f_walking(), initialiseEEPROM()

//== NEW ====================================================
/*
#include "pinsPM20.h"
#include "varsConsPM20.h"
#include "initPM20.h"
*/
//== CHANGED ================================================
/*
Pin declarations moved to Pins.h which opens in a Tab
Variables and Constant declarations moved to VarsCons.h which opens in a Tab
The unchanging initialisation in Setup() moved to Init.h which opens in a Tab
Functions moved to .ino files which open in Tabs
*/
//== INCLUDES ===============================================
#include "pinsPM20.h"
#include "varsConsPM20.h"
#include <EEPROM.h>           // to read and write internal EEPROM
  // EEPROM.write(address,bytevalue)
  // bytevalue =EEPROM.read(address)
//== SETUP ==================================================
void setup() 
{ 
  #include "initPM20.h" //Initialise various things
  
  //now configure Bambino
  f_forwards =1;    // set default
  rollspeed =1;     // start off slow
  pacespeed =1;     // start off slow
  rollspeedmax =6;  // quite fast 
  pacespeedmax =6;  // quite fast
}
//== LOOP ===================================================
void loop()
{ 
  Wander();
}  
//===========================================================












