// B_PM21_PlayTime.ino
/*
Now Bambino can move about and sense obstacles we can get it come alive.
Two new routines rock() and shuffle() allow loop() to have Bambino Wander 
for a few steps then rock then wander again then shuffle.
Routines whee() and whoa() give Bambino a better voice.
Build some routines for yourself.
*/

char thisrobot[] ="BambinoPM ";
char thisprog[]  ="B_PM21_PlayTime";
// Binary sketch size: 7,176 bytes (of a 30,720 byte maximum)

//== SECTIONS ===============================================
//== NEW 
//== CHANGED
//== INCLUDES
//     pinsPM20.h     - pin usage
//     varsConsPM20.h - EEPROM addresses, general, Ultrasonic module HC-SR04
//     EEPROM.h     - the Arduino EEPROM library
//== EEPROM
//== SETUP
//     initPM20.h     - Initialise various things
//== LOOP
//== FUNCTIONS
//   rock(repeats)
//   shuffle(repeats)
//   whee()   // make rising sound
//   whoa()   // make falling sound
//TAB FILES
//   actsPM21.ino
//     ACTS - WanderSteps(steps), Wander()
//   behaviourPM20.ino
//     BEHAVIOUR MODIFIERS - speeddown(), speedup()
//     BEHAVIOUR RULES - avoidUObjects(), avoidObstacles(), avoidHoles()
//   sensePM20.ino
//     SENSE - dist_HC_SR04()
//           - readWhiskers(), QueryObstacle(), showWhiskerLightLevels() 
//           - readBeard(), QueryEdge(), showBeardLightLevels() 
//   movement20.ino
//     META MOVEMENT - turnAway(), turnMemory(), turnOther() 
//     MOVEMENT - forward(), backward(), turnRight(), turnLeft(),
//                rollCentre(), rollRight(), rollLeft(), 
//                paceCentre(), paceRightForward(), paceLeftForward(), paceRightBackward(), paceLeftBackward(), 
//                rollmove(), pacemove()
//   servos20.ino
//     SERVOS - ServosInitC()
//   hardware20.ino
//     HARDWARE - ServoPulse(byte Spin, int Spulse), beep(byte beeppin,int beepfreqHz,long beeptime)
//   system20.ino
//     SYSTEM - toggle_f_walking(), initialiseEEPROM()

//== NEW ====================================================
/*
actsPM21.ino - void WanderSteps(int repeat)     // accepts a repeat parameter, calls Wander()
void rock(repeats)
void shuffle(repeats)
void whee()   // make rising sound
void whoa()   // make falling sound 
*/
//== CHANGED ================================================
/*
loop()
*/
//== INCLUDES ===============================================
#include "pinsPM20.h"
#include "varsConsPM20.h"
#include <EEPROM.h>           // to read and write internal EEPROM
  // EEPROM.write(address,bytevalue)
  // bytevalue =EEPROM.read(address)
//== VARIABLES and CONSTANTS ================================         
//== SETUP ==================================================
void setup() 
{ 
  #include "initPM20.h" //Initialise various things
  
  //now configure Bambino
  f_forwards =1;    // set default
  rollspeed =1;     // start off slow
  pacespeed =1;     // start off slow
  rollspeedmax =6;  // quite fast 
  pacespeedmax =6;  // quite fast
}
//== LOOP ===================================================
void loop()
{ 
  WanderSteps(5); 
  rollspeed =2;     // for example
  whoa();
  rock(2);
  WanderSteps(3);
  whee();
  shuffle(3);  
}  
//== FUNCTIONS ==============================================
void rock(int steps)
{ while(steps--)
  { rollRight(); rollLeft(); }
}
//----------------------------
void shuffle(int steps)
{ while(steps--)  
  { paceRightForward(); paceRightBackward(); }
}
//----------------------------
void whee()   // make rising sound
{ for (int i=500; i<5000; i+=100) 
    { beep(voicePin,i,10); }
}	
//----------------------------
void whoa()   // make falling sound
{ for (int i=5000; i>500; i-=100) 
    { beep(voicePin,i,10); }
}	
//===========================================================












