// B_PM22_BeAlive.ino
/*
By using a switch statement with a random number as the case variable
Bambino can be made to do sequences which don't repeat and
it seems to have a mind of its own and looks alive. 
Note RightStep(repeat) and LeftStep(repeat) which help it from getting stuck.
Experiment with different values for the min and max in random().
You could write similar functions to RightStep(repeat) and LeftStep(repeat) 
for turnAway(), turnMemory(), turnOther() and add them to the switch list.
If you do, remember to increase the random range for the case variable.
*/

char thisrobot[] ="BambinoPM ";
char thisprog[]  ="B_PM22_BeAlive";
// Binary sketch size: 8,030 bytes (of a 30,720 byte maximum)

//== SECTIONS ===============================================
//== NEW 
//== CHANGED
//== INCLUDES
//     pinsPM20.h     - pin usage
//     varsConsPM20.h - EEPROM addresses, general, Ultrasonic module HC-SR04
//     EEPROM.h       - the Arduino EEPROM library
//== EEPROM
//== SETUP
//     initPM20.h     - Initialise various things
//== LOOP
//== FUNCTIONS
//   RightStep(int repeat)   //turnRight for repeat steps
//   LeftStep(int repeat)   //turnLeft for repeat steps
//   rock(repeats)
//   shuffle(repeats)
//   whee()   // make rising sound
//   whoa()   // make falling sound
//TAB FILES
//   actsPM21.ino
//     ACTS - WanderSteps(steps), Wander()
//   behaviourPM20.ino
//     BEHAVIOUR MODIFIERS - speeddown(), speedup()
//     BEHAVIOUR RULES - avoidUObjects(), avoidObstacles(), avoidHoles()
//   sensePM20.ino
//     SENSE - dist_HC_SR04()
//           - readWhiskers(), QueryObstacle(), showWhiskerLightLevels() 
//           - readBeard(), QueryEdge(), showBeardLightLevels() 
//   movement20.ino
//     META MOVEMENT - turnAway(), turnMemory(), turnOther() 
//     MOVEMENT - forward(), backward(), turnRight(), turnLeft(),
//                rollCentre(), rollRight(), rollLeft(), 
//                paceCentre(), paceRightForward(), paceLeftForward(), paceRightBackward(), paceLeftBackward(), 
//                rollmove(), pacemove()
//   servos20.ino
//     SERVOS - ServosInitC()
//   hardware20.ino
//     HARDWARE - ServoPulse(byte Spin, int Spulse), beep(byte beeppin,int beepfreqHz,long beeptime)
//   system20.ino
//     SYSTEM - toggle_f_walking(), initialiseEEPROM()

//== NEW ====================================================
/*
void RightStep(int repeat)   //turnRight for repeat steps
void LeftStep(int repeat)   //turnLeft for repeat steps
*/
//== CHANGED ================================================
/*
loop()
*/
//== INCLUDES ===============================================
#include "pinsPM20.h"
#include "varsConsPM20.h"
#include <EEPROM.h>           // to read and write internal EEPROM
  // EEPROM.write(address,bytevalue)
  // bytevalue =EEPROM.read(address)
//== VARIABLES and CONSTANTS ================================           
//== SETUP ==================================================
void setup() 
{ 
  #include "initPM20.h" //Initialise various things
  
  //now configure Bambino
  f_forwards =1;    // set default
  rollspeed =1;     // start off slow
  pacespeed =1;     // start off slow
  rollspeedmax =4;  // quite fast 
  pacespeedmax =6;  // quite fast
}
//== LOOP ===================================================
void loop()
{ switch (random(9))
  { case 0: WanderSteps(random(10)); break;
    case 1: rock(random(3)); break;
    case 2: shuffle(random(3)); break;
    case 3: whoa(); break;
    case 4: whee(); break;
    case 5: rollspeed =random(1,3); break;
    case 6: pacespeed =random(1,6); break;
    case 7: RightStep(random(6)); break;
    case 8: LeftStep(random(6)); break;
  }
}  
//== FUNCTIONS ==============================================
void RightStep(int repeat)   //turnRight for repeat steps
{ while(repeat--)
  { turnRight(); }
}
//----------------------------
void LeftStep(int repeat)   //turnLeft for repeat steps
{ while(repeat--)
  { turnLeft(); }
}
//----------------------------
void rock(int repeat)
{ while(repeat--)
  { rollRight(); rollLeft(); }
}
//----------------------------
void shuffle(int repeat)
{ while(repeat--)  
  { paceRightForward(); paceRightBackward(); }
}
//----------------------------
void whee()   // make rising sound
{ for (int i=500; i<5000; i+=100) 
    { beep(voicePin,i,10); }
}	
//----------------------------
void whoa()   // make falling sound
{ for (int i=5000; i>500; i-=100) 
    { beep(voicePin,i,10); }
}	
//===========================================================








