// Bambino
// TAB FILE > sensePM36.ino

//     SENSE - dist_HC_SR04()
//           - readWhiskers(), QueryObstacle(), showWhiskerLightLevels() 
//           - readBeard(), QueryEdge(), showBeardLightLevels()
//           - readLight()

//== SENSE ==================================================
byte dist_HC_SR04()              //get range to object
{ int UStime;
  pinMode(pingPin,OUTPUT);
  digitalWrite(pingPin,HIGH);
  delayMicroseconds(5);                  // 3 and below gives erratic results
  digitalWrite(pingPin,LOW);
  pinMode(pingPin,INPUT);
  UStime =pulseIn(pingPin,HIGH,UStimeout);  // UStimeout is set to 20000 =20ms
  if (UStime==0) {UStime =UStimeout;}    // timeout returns 0, so make it a big distance
  //Serial.print(UStime);
  if (f_cm) {return UStime/uscm;} else {return UStime/usinch;}
}	  
//-----------------------------------------------------------     
void readLight()  // read ambient Light levels   0 - 1023
//reading every ms for 8ms ensures that a peak (or near peak) of 100Hz flicker is captured.   
{ digitalWrite(RwhiskerLEDpin_, HIGH);             // whisker LED OFF in case it was on
  digitalWrite(LwhiskerLEDpin_, HIGH);             // whisker LED OFF in case it was on
  int readings =8;
  LeyeLight =0; ReyeLight =0;
  while(readings--)
  { ReyeLight =max(ReyeLight,analogRead(RwhiskerSensePin));  // measure light level
    LeyeLight =max(LeyeLight,analogRead(LwhiskerSensePin));  // measure light level
    delay(1);
  }
  Serial.print("(");	
  Serial.print(LeyeLight);
  Serial.print(",");
  Serial.print(ReyeLight);
  Serial.print(")");
}
//----------------------------------------------------------- 
void readWhiskers()         //16ms total delay,    see also //== ALTERNATIVE CODE for readBeard()
//reading every ms for 8ms ensures that a peak (or near peak) of 100Hz flicker is captured.   
{ LwhiskerLight =0;  RwhiskerLight =0;  LwhiskerDark =0;  RwhiskerDark =0;
  digitalWrite(RwhiskerLEDpin_, LOW);                              // whisker LED ON
  digitalWrite(LwhiskerLEDpin_, LOW);                              // whisker LED ON
  int readings =8;
  while(readings--)
  { RwhiskerLight =max(RwhiskerLight,analogRead(RwhiskerSensePin));  // measure light level
    LwhiskerLight =max(LwhiskerLight,analogRead(LwhiskerSensePin));  // measure light level
    delay(1);
  }	
  digitalWrite(RwhiskerLEDpin_, HIGH);                             // whisker LED OFF
  digitalWrite(LwhiskerLEDpin_, HIGH);                             // whisker LED OFF
  readings =8;
  while(readings--)
  { RwhiskerDark =max(RwhiskerDark,analogRead(RwhiskerSensePin));  // measure light level
    LwhiskerDark =max(LwhiskerDark,analogRead(LwhiskerSensePin));  // measure light level
    delay(1);
  }	
  RwhiskerDifference =RwhiskerLight-RwhiskerDark;
  LwhiskerDifference =LwhiskerLight-LwhiskerDark;
  showWhiskerLightLevels();
}
//----------------------------
void queryObstacles()
// calculate difference in light value with LED on and LED off and compare to obstacleThreshold
{ if ( RwhiskerDifference > obstacleThresholdRight ) {f_obstacleRight =1;} else {f_obstacleRight =0;}
  if ( LwhiskerDifference > obstacleThresholdLeft )  {f_obstacleLeft =1;} else {f_obstacleLeft =0;}
}
//----------------------------
void showWhiskerLightLevels()     //print light levels to Serial Monitor
{ //39ms 
  Serial.println();
  Serial.print("whisker Left=");
  Serial.print(LwhiskerLight);
  Serial.print(",");
  Serial.print(LwhiskerDark);
  Serial.print(",");
  Serial.print(LwhiskerLight -LwhiskerDark);
  Serial.print(", Right=");
  Serial.print(RwhiskerLight);
  Serial.print(",");
  Serial.print(RwhiskerDark);
  Serial.print(",");
  Serial.println(RwhiskerLight -RwhiskerDark);
}  
//----------------------------
void readBeard()         //16ms total delay,    see also //== ALTERNATIVE CODE
{ LbeardLight =0;  RbeardLight =0;  LbeardDark =0;  RbeardDark =0;
  digitalWrite(RbeardLEDpin_, LOW);                         // beard LED ON
  digitalWrite(LbeardLEDpin_, LOW);                         // beard LED ON
  int readings =8;
  while(readings--)
   {LbeardLight =max(LbeardLight,analogRead(LbeardSensePin)); // measure light level
    RbeardLight =max(RbeardLight,analogRead(RbeardSensePin)); // measure light level
    delay(1);
   }	
  digitalWrite(RbeardLEDpin_, HIGH);                        // beard LED OFF
  digitalWrite(LbeardLEDpin_, HIGH);                        // beard LED OFF
  readings =8;
  while(readings--)
   {RbeardDark =max(RbeardDark,analogRead(RbeardSensePin));   // measure dark level  
    LbeardDark =max(LbeardDark,analogRead(LbeardSensePin));   // measure dark level
    delay(1);
   }	
  RbeardDifference =RbeardLight-RbeardDark;
  LbeardDifference =LbeardLight-LbeardDark;
  showBeardLightLevels();
}
//----------------------------
void queryEdge()
// calculate difference in light value with LED on and LED off and compare to holeThreshold
{ if ( RbeardDifference < holeThresholdRight ) {f_holeRight =1;} else {f_holeRight =0;}
  if ( LbeardDifference < holeThresholdLeft )  {f_holeLeft =1;} else {f_holeLeft =0;}
}
//----------------------------
void showBeardLightLevels()     //print light levels to Serial Monitor
{ //39ms
  Serial.println();
  Serial.print("Beard Left=");
  Serial.print(LbeardLight);
  Serial.print(",");
  Serial.print(LbeardDark);
  Serial.print(",");
  Serial.print(LbeardLight -LbeardDark);
  Serial.print(", Right=");
  Serial.print(RbeardLight);
  Serial.print(",");
  Serial.print(RbeardDark);
  Serial.print(",");
  Serial.println(RbeardLight -RbeardDark);
} 
//== ALTERNATIVE CODE =======================================
/*
void readBeard()         //8ms total delay - not as consistent readings
{ LbeardLight =0;	RbeardLight =0;	LbeardDark =0;	RbeardDark =0;
  int readings =8;
  while(readings--)
   {LbeardDark =max(LbeardDark,analogRead(LbeardSensePin));   // measure dark level  
    digitalWrite(LbeardLEDpin_, LOW);                         // beard LED ON
    LbeardLight =max(LbeardLight,analogRead(LbeardSensePin)); // measure light level
    digitalWrite(LbeardLEDpin_, HIGH);                        // beard LED OFF
    RbeardDark =max(RbeardDark,analogRead(RbeardSensePin));   // measure dark level
    digitalWrite(RbeardLEDpin_, LOW);                         // beard LED ON
    RbeardLight =max(RbeardLight,analogRead(RbeardSensePin)); // measure light level
    digitalWrite(RbeardLEDpin_, HIGH);                        // beard LED OFF
	delay(1);
   }	
  RbeardDifference =RbeardLight-RbeardDark;
  LbeardDifference =LbeardLight-LbeardDark;
  showBeardLightLevels();
}
*/
//===========================================================





